      module  eigen_gev
!--------+---------+---------+---------+---------+---------+---------+---------|
         use eigen_libs
         implicit NONE
!--------+---------+---------+---------+---------+---------+---------+---------|
      interface
!--------+---------+---------+---------+---------+---------+---------+---------|
         subroutine  KMATH_EIGEN_GEV( n, a, lda, b, ldb, w, z, ldz,
     &                                mode, solver, info )
!
!   integer(In)    n        :: Matrix dimension
!   real(8)(InOut) a(lda,*) :: An input matrix to be solved for GEV of LHS.
!                              A is real symmetric matrix,
!                              and an upper triangular format distributed on
!                              a 2D cyclic-cyclic fashion.
!                              On exit, data is destroyed.
!   integer(In)    lda      :: Leading Dimension of the array A
!   real(8)(InOut) b(ldb,*) :: An input matrix to be solved for GEV of RHS.
!                              B is real positive definite symmetric matrix,
!                              and an upper triangular format distributed on
!                              a 2D cyclic-cyclic fashion.
!                              The transformation factor to SEP,
!                                 eig.vec(B) * diag(eig.val(B))^{-1/2}.
!                              is returned on exit.
!   integer(In)    ldb      :: Leading Dimension of the array B
!   real(8)(Out)   w(*)     :: An array where eigenvalues computed are stored
!   real(8)(Out)   z(ldz,*) :: An output matrix where eigenvectors are stored
!   integer(In)    ldz      :: Leading Dimension of the array Z
!   character*1(In)  mode   :: Computing mode
!   character*1(In)  solver :: Solver
!   integer(Out)   info     :: Status information
!
         integer, intent(In)    ::  n, lda, ldb, ldz
         real(8), intent(InOut) ::  a(*), b(*)
         real(8), intent(InOut) ::  w(*), z(*)
         character*1, intent(In), optional ::  mode, solver
         integer, intent(Out), optional ::  info

         end subroutine  KMATH_EIGEN_GEV
!--------+---------+---------+---------+---------+---------+---------+---------|
      end interface
!--------+---------+---------+---------+---------+---------+---------+---------|
      contains
!--------+---------+---------+---------+---------+---------+---------+---------|
         subroutine  eigen_gev_s ( n, a, lda, b, ldb, w, z, ldz,
     &                                mode )
         integer, intent(In)    ::  n, lda, ldb, ldz
         real(8), intent(InOut) ::  a(*), b(*)
         real(8), intent(InOut) ::  w(*), z(*)
         character*1, intent(In), optional ::  mode

            call KMATH_EIGEN_GEV( n, a, lda, b, ldb, w, z, ldz,
     &                            mode=mode, solver='S' )

         end subroutine  eigen_gev_s
         subroutine  eigen_gev_sx( n, a, lda, b, ldb, w, z, ldz,
     &                                mode )
         integer, intent(In)    ::  n, lda, ldb, ldz
         real(8), intent(InOut) ::  a(*), b(*)
         real(8), intent(InOut) ::  w(*), z(*)
         character*1, intent(In), optional ::  mode

            call KMATH_EIGEN_GEV( n, a, lda, b, ldb, w, z, ldz,
     &                            mode=mode, solver='X' )

         end subroutine  eigen_gev_sx
!--------+---------+---------+---------+---------+---------+---------+---------|
      end module  eigen_gev

